<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\StyleService;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;
use Service\Traits\TResponse;

class Style extends Crud {

    use TResponse;

	protected $dataSourceName = 'style';

	/** @var StyleService */
    protected $styleService;

	public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, $styleService) {
		parent::__construct($mongoManager, $deleteCascadeService);
        $this->styleService = $styleService;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->orderFields = [];
	}

    public function findNext(DTO\Request $request, DTO\Response $response) {
        try{
            $params = $request->getParameters();
            $style = $this->styleService->getStyleGeneral($params);
            $style = Utilities::parseDataSourceZeedhiNext($style);
            $this->sendResponse(Utilities::paginationAndOrderINPhpNext($style, $params), 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    private function validateParameters($style, $params){
        if (array_key_exists("order", $params)) {
            $periodData = $style['data'];
            $order = $params['order'];
            
            foreach($order as &$orderField) {
                $orderField = explode(".", $orderField);
            }
            
            $this->orderFields = $order;
            usort($periodData, array($this, 'comparator'));
            $style['data'] = $periodData;
        }

        $limit = -1;
        if(array_key_exists("limit", $params)){
            $limit = intval($params['limit']);
            $style['pagination']['limit'] = $limit;
            $styleData = array_chunk($style['data'], $limit);
        }

        $page = 1;
        if(array_key_exists("page", $params)){
            $page = intval($params['page']);
            $style['pagination']['page'] = $page;
            if($limit != -1 && sizeof($styleData) >= $page) $style['data'] = $styleData[$page-1];
        }

        return $style;
    }

    private function comparator($object1, $object2)
    {
        $orderList = $this->orderFields;
        foreach($orderList as $order) {
            $orderResult = ($order[1] == 'asc') ? $this->comparatorAsc($object1, $object2, $order[0]) : $this->comparatorDesc($object1, $object2, $order[0]);
            if ($orderResult)
                return $orderResult;
        }
        return 0;
    }

    private function comparatorAsc($object1, $object2, $field)
    {
        return strcmp(strtolower($object1[$field]), strtolower($object2[$field]));
    }

    private function comparatorDesc($object1, $object2, $field)
    {
        return strcmp(strtolower($object2[$field]), strtolower($object1[$field]));
    }
}
