import { ChartViewService } from "@/Services/ChartViewService";
import { ChartViewFactory } from '@/Services/ChartView/ChartViewFactory';
import { Metadata, FormatterParserProvider } from "@zeedhi/core";
import { Col, Form, ISelect, IFormComponent, Button, Modal, ModalService } from "@zeedhi/common";
import { ChartViewController } from '../../controllers/ChartViewController';
import { DashboardController } from '../../controllers/DashboardController';
import { Utilities } from '../Utilities';

export class ZmartChartsApex {
    public charts: any;
    public chartMetadata: any = {};
    public chartContext: any = {};
    public chartInstance: any = {};

    public tela: any = {};
    public chartViewController: any;
    private factMaskFunction: any;
    private dimensionMaskFunction: any;
    private enableDrillEvents: Boolean = false;

    private dashboardController: DashboardController = new DashboardController();

    public static isDashboard: any = (window.location.pathname !== "/chartView");

    public static build(chartMetadata: any, chartViewController: any, component: any) {
        const instacia = new this();
        instacia.setChartMetadata(chartMetadata);
        if (chartMetadata[0].name !== 'notFound') {
            if (!chartViewController) {
                chartViewController = new ChartViewController();
            }
            instacia.setChartViewController(chartViewController);
            instacia.create(chartMetadata);
        } else {
            instacia.charts = chartMetadata;
        }
        instacia.setTela(component);
        return instacia;
    }

    public getCharts() {
        return this.charts;
    }

    public setTela(tela: any) {
        this.tela = tela;
    }

    public async drillCardList(widget: any, {component}: any) {
        console.log(component);
        const card = component.parent.children.find((childrenCard: any) => childrenCard.name === component.name);
        const fieldDrill = widget[widget.dimensionProperty][0];
        const valueDrill = card.cardDrillValue;
        this.doDrillCard(widget, fieldDrill, valueDrill);
    }

    public async drillCard(args: any) {
        const card = args.component.children[1];
        const widget = card.widget;
        if (widget && widget.hasDrillDown) {
            const fieldDrill = widget[widget.dimensionProperty];
            const valueDrill = card.cardData[fieldDrill];
            this.doDrillCard(widget, fieldDrill, valueDrill);
        }
    }

    public doDrillCard(widget: any, fieldDrill: any, valueDrill: any) {
        const drillDownView = widget.drillDownView;
        if (drillDownView) {
            let drillDownFilters: any = {};
            drillDownFilters = {...drillDownFilters, ...widget.parameters.drillDownFilters};
            drillDownFilters[fieldDrill] = valueDrill;
            if (widget.parameters.period) {
                drillDownFilters.period = widget.parameters.period;
            }
            const structureId = widget.parameters.structureId[0];
            const structureName = widget.parameters.structureName[0];
            this.openDrill(drillDownView, drillDownFilters, structureId, structureName);
        }
    }

    public async drillGrid(widget: any, args: any) {
        if (widget.hasDrillDown && widget.drillDownDimensions) {
            const drillDownView = widget.drillDownView;
            if (drillDownView) {
                let drillDownFilters: any = {};
                drillDownFilters = {...drillDownFilters, ...widget.parameters.drillDownFilters};
                if (widget.parameters && widget.parameters.period) {
                    drillDownFilters.period = widget.parameters.period;
                }
                widget.drillDownDimensions.forEach(function (fieldDrill: any) {
                    const valueDrill = args.row[fieldDrill];
                    drillDownFilters[fieldDrill] = valueDrill;
                });
                this.openDrill(drillDownView, drillDownFilters);
            }
        }
    }

    public async drillPeriodCompare(widget: any, { row }: any) {
        if (widget.hasDrillDown) {
            const drillDownView = widget.drillDownView;
            if (drillDownView) {
                const fieldDrill = widget.periodsDimension;
                const valueDrill = row.label;
                let drillDownFilters: any = {};
                drillDownFilters = {...drillDownFilters, ...widget.parameters.drillDownFilters};
                drillDownFilters[fieldDrill] = valueDrill;
                const periodValue = row.period.split(" - ");
                drillDownFilters.period = {
                    "start": periodValue[0],
                    "end": periodValue[1]
                };
                this.openDrill(drillDownView, drillDownFilters);
            }
        }
    }

    public async drillCrosstab(widget: any, { component, value, labels }: any) {
        if (widget.hasDrillDown) {
            const drillDownView = widget.drillDownView;
            let drillDownFilters: any = {};
            drillDownFilters = {...drillDownFilters, ...widget.parameters.drillDownFilters};
            labels.forEach(function (filter: any) {
                const fieldDrill = filter.field.name;
                const valueDrill = filter.value;
                drillDownFilters[fieldDrill] = valueDrill;
            });
            if (widget.parameters && widget.parameters.period) {
                drillDownFilters.period = widget.parameters.period;
            }
            this.openDrill(drillDownView, drillDownFilters);
        }
    }

    public async drillEChartsBar(widget: any, { component, param }: any) {
        if (widget.hasDrillDown) {
            const fieldDrill = widget.chartXAxisField;
            const valueDrill = param.name;
            const drillDownView = widget.drillDownView;
            this.openDrillEcharts(widget, fieldDrill, valueDrill, drillDownView);
        }
    }

    public async drillECharts(widget: any, { component, param }: any) {
        if (widget.hasDrillDown) {
            const dataPropertyName: any = {
                "gauge": "gaugeData",
                "pie": "pieData",
            };
            const fieldDrill = widget.chartXAxisField;
            const valueDrill = widget[dataPropertyName[widget.chartType]][widget.chartXAxisField];
            const drillDownView = widget.drillDownView;
            this.openDrillEcharts(widget, fieldDrill, valueDrill, drillDownView);
        }
    }

    private async openDrillEcharts(widget:any, fieldDrill: any, valueDrill: any, drillDownView: any) {
        if (drillDownView) {
            let drillDownFilters: any = {};
            drillDownFilters = {...drillDownFilters, ...widget.parameters.drillDownFilters};
            if (widget.parameters && widget.parameters.period) {
                drillDownFilters.period = widget.parameters.period;
            }
            drillDownFilters[fieldDrill] = valueDrill;
            this.openDrill(drillDownView, drillDownFilters);
        }
    }

    public async drillApexChart(widget: any, event: any, chartContext: any, component: any) {
        const fieldDrill = component.config.xaxis.field;
        const valueDrill = component.config.xaxis.categories[component.dataPointIndex];
        const drillDownView = component.config.drillDownView;
        if (drillDownView) {
            let drillDownFilters: any = {};
            drillDownFilters = {...drillDownFilters, ...widget.parameters.drillDownFilters};
            drillDownFilters[fieldDrill] = valueDrill;
            if (widget.parameters.period) {
                drillDownFilters.period = widget.parameters.period;
            }
            this.openDrill(drillDownView, drillDownFilters);
        }
    }

    public async goBackDrill() {
        const previousDrillView = this.chartViewController.drillDownViewStack.pop();
        this.openDrill(previousDrillView, this.chartViewController.drillDownFiltersList[previousDrillView], undefined, undefined, true);
    }

    private async openDrill(drillDownView: any, drillDownFilters: any, structureId?: any, structureName?: any, isBack?: Boolean) {
        if (!isBack) {
            const thisChartViewId = this.chartViewController.viewId;
            this.chartViewController.drillDownViewStack.push(thisChartViewId);
            this.chartViewController.drillDownFiltersList[thisChartViewId] = this.chartViewController.drillDownFilters;
        }
        ZmartChartsApex.isDashboard = (window.location.pathname !== "/chartView");
        this.openDrillDashboard(drillDownView, drillDownFilters, structureId, structureName);
    }

    public async openDrillDashboard(drillDownView: any, drillDownFilters: any, structureId?: any, structureName?: any) {
        localStorage.setItem('drillDownView', drillDownView);
        localStorage.setItem('drillDownFilter', JSON.stringify(drillDownFilters));
        localStorage.setItem('drillDownStructure', JSON.stringify({
            "name": structureName,
            "id": structureId,
        }));
        this.dashboardController.openDrillModal();
    }

    public getFactMaskFunction() {
        return this.factMaskFunction;
    }

    public factFormatter(value: any, apex: any) {
        let valueMask = value;
        if (!isNaN(value) && value != null) {
            valueMask = this.factMaskFunction(value);
        }
        return valueMask;
    }

    public dimensionFormatter(value: any, apex: any) {
        let valueMask = value;
        valueMask = this.dimensionMaskFunction(value);
        return valueMask;
    }

    public dataLabelsFormatter(value: any, apex: any) {
        if (apex.w.config.chart.formatValue) {
            if (apex.w.config.dataLabels.showAsPercent) {
                value = Utilities.roundFloat(value);
            } else {
                value = apex.w.config.series[apex.seriesIndex];
            }
        }
        let valueMask = value;
        if (!isNaN(value)) {
            if (apex.w.config.chart.stacked) {
                if (apex.seriesIndex === ((apex.w.config.series.length) - 1)) {
                    valueMask = this.factMaskFunction(apex.w.globals.stackedSeriesTotals[apex.dataPointIndex]);
                } else {
                    valueMask = "";
                }
            } else if (value != null) {
                valueMask = this.factMaskFunction(value);
            }
        }
        return valueMask;
    }

    public setChartContext(chartContext: any) {
        this.chartContext = chartContext;
    }

    public mounted(chartContext: any, config: any) {
        this.setChartContext(chartContext);
    }

    public setListCardFormatterFunctions(card: any) {
        if (card[0].widget.fieldValueMask) {
            const cardRows = card[0].children.filter((row: any) => row.isCardsRow);
            cardRows.forEach((row: any) => {
                row.children.forEach((cardComponent: any) => {
                    const cardValueElement = cardComponent.children[1];
                    const cardValue = cardValueElement.text.replace(/<\/?[^>]+(>|$)/g, '');
                    const maskedValue = ChartViewService.getMaskFunction(card[0].widget.fieldValueMask)(Number(cardValue));
                    cardValueElement.text = cardValueElement.text.replace(cardValue, maskedValue);
                });
            });
        }
    }

    public setCardFormatterFunctions(card: any) {
        const cardValueElementList = [
            card[0].children[1]
        ];
        const cardMaskList = [
            card[0].widget.cardMask
        ];
        if (card[0].isCardDetail) {
            cardValueElementList.push(
                card[0].children[2].children[0].children[1],
                card[0].children[2].children[1].children[1]
            );
            cardMaskList.push(
                card[0].widget.cardMask2,
                card[0].widget.cardMask3
            );
        }
        cardValueElementList.forEach(function (cardValueElement, i) {
            const cardValue = cardValueElement.text.replace(/<\/?[^>]+(>|$)/g, '');
            const maskedValue = ChartViewService.getMaskFunction(cardMaskList[i])(Number(cardValue));
            cardValueElement.text = cardValueElement.text.replace(cardValue, maskedValue);
        });
    }

    public setEChartsFormatterFunctions(chart: any) {
        if (chart[0].options.xAxis.axisLabel && chart[0].options.xAxis.maskFunction) {
            chart[0].options.xAxis.axisLabel.formatter = ChartViewService.getMaskFunction(chart[0].options.xAxis.maskFunction);
        }
        if (chart[0].options.yAxis.axisLabel && chart[0].options.yAxis.maskFunction) {
            chart[0].options.yAxis.axisLabel.formatter = ChartViewService.getMaskFunction(chart[0].options.yAxis.maskFunction);
        }
    }

    public setGridFormatterFunctions(charts: any) {
        if (charts && charts[0] && charts[0].columns) {
            charts[0].columns.forEach((column: any) => {
                if (column.mask) {
                    FormatterParserProvider.registerFormatter(column.componentProps.component, ChartViewService.getMaskFunction(column.mask));
                }
            });
        }
    }

    public setFormatterFunctions(charts: any) {
        let factAxis = "yaxis";
        let dimensionAxis = "xaxis";
        if (charts[0].invertedAxis) {
            factAxis = "xaxis";
            dimensionAxis = "yaxis";
        }
        this.dimensionMaskFunction = ChartViewService.getMaskFunction(charts[0].options.xaxis.mask);
        charts[0].options[dimensionAxis].labels.formatter = this.dimensionFormatter.bind(this);

        if (charts[0].options[factAxis].length) {
            for (let i = 0; i < charts[0].options[factAxis].length; i += 1) {
                this.factMaskFunction = ChartViewService.getMaskFunction(charts[0].options.yaxis[i].mask);
                charts[0].options[factAxis][i].labels.formatter = this.factFormatter.bind(this);
            }
            charts[0].options.dataLabels.formatter = ChartViewService.getMaskFunctionDynamic;
        } else {
            this.factMaskFunction = ChartViewService.getMaskFunction(charts[0].options.yaxis.mask);
            charts[0].options[factAxis].labels.formatter = this.factFormatter.bind(this);
            charts[0].options.dataLabels.formatter = this.dataLabelsFormatter.bind(this);
        }
    }

    public create(charts: any) {
        this.chartInstance = ChartViewFactory.getInstanceChartType(charts[0].viewTypeId);
        charts = this.chartInstance.getCharts(charts);
        this.enableDrillEvents = true;
        charts = this.chartInstance.setDrill(charts, this);
        this.charts = charts;
    }

    public setChartMetadata(chartMetadata: any) {
        this.chartMetadata = chartMetadata;
    }
    public setChartViewController(chartViewController: any) {
        this.chartViewController = chartViewController;
    }

    public formButtonEditClickApplyView() {
        const formViewEdit = Metadata.getInstance<Form>('form-view-edit');
        const currentRow = formViewEdit.value;
        this.charts[0].options.title.text = currentRow.name;

        const type = currentRow.serieType;
        this.charts[0].options.plotOptions.bar.horizontal = currentRow.chartInvertedAxes;
        this.charts[0].options.dataLabels.offsetX = currentRow.offsetX;
        this.charts[0].options.dataLabels.offsetY = currentRow.offsetY;

        if (this.charts[0].chartType !== type) {
            this.charts[0].chartType = type;
        } else {
            this.tela.childrenInstances[1].updateChart(this.charts[0].options);
        }
        this.tela.children = [];
        this.tela.children = this.charts;
    }

    public formButtonEditClickCloseView() {
        ChartViewController.closeEditView();
        this.chartContext.destroy();
        this.tela.childrenInstances = [];
        this.chartViewController.buildWidgetByViewId(this.tela, this.charts[0].id, {}).then();
    }

    public setEventsButton() {
        const formButtonEdit = Metadata.getInstance<Form>('form-button-edit');
        const buttonApplyView = formButtonEdit.getChildInstance<Button>('applyView');
        const buttonCloseView = formButtonEdit.getChildInstance<Button>('closeView');
        buttonApplyView.events.click = this.formButtonEditClickApplyView.bind(this);
        buttonCloseView.events.click = this.formButtonEditClickCloseView.bind(this);
    }
    public formViewEditChangeType() {
        const formViewEdit = Metadata.getInstance<Form>('form-view-edit');
        const currentRow = formViewEdit.value;
        const fieldType = formViewEdit.getChildInstance<ISelect>('type');
        if (fieldType.datasource && fieldType.datasource.currentRow) {
            const fieldTypeCurrentRow = <any>fieldType.datasource.currentRow;
            currentRow.serieType = fieldTypeCurrentRow.serieType ? fieldTypeCurrentRow.serieType : fieldTypeCurrentRow.name;
            currentRow.chartInvertedAxes = !!fieldTypeCurrentRow.chartInvertedAxes;
            currentRow.offsetX = fieldTypeCurrentRow.offsetX ? fieldTypeCurrentRow.offsetX : null;
            currentRow.offsetY = fieldTypeCurrentRow.offsetY ? fieldTypeCurrentRow.offsetY : null;
            // ChartViewController.setFieldsEditView(fieldTypeCurrentRow);
        }
    }

    public async onclickEditCharts(chart: any, options: any, e: any) {
        ChartViewController.openEditView();

        this.setEventsButton();

        const formViewEdit = Metadata.getInstance<Form>('form-view-edit');
        const currentRow = formViewEdit.value;

        currentRow.name = this.charts[0].options.title.text;

        const fieldType = formViewEdit.getChildInstance<ISelect>('type');
        if (fieldType.events) {
            fieldType.events.change = this.formViewEditChangeType.bind(this);
        }
        if (fieldType.datasource) {
            await fieldType.datasource.get();
            fieldType.setValue(this.charts[0].viewTypeId);
            this.formViewEditChangeType();
        }
    }
}
